package net.knarcraft.questmobspawns.util;

import net.knarcraft.questmobspawns.QuestMobSpawns;
import org.bukkit.Location;
import org.bukkit.NamespacedKey;
import org.bukkit.entity.Entity;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;

public class QuestMobHelper {

    private static final NamespacedKey questMobKey = new NamespacedKey(QuestMobSpawns.getPlugin(), "questMob");
    private static final NamespacedKey questIdKey = new NamespacedKey(QuestMobSpawns.getPlugin(), "questId");
    private static final NamespacedKey stageIdKey = new NamespacedKey(QuestMobSpawns.getPlugin(), "stageId");
    private static final NamespacedKey mobIndexKey = new NamespacedKey(QuestMobSpawns.getPlugin(), "mobIndex");
    private static final NamespacedKey killZoneLocationKey = new NamespacedKey(QuestMobSpawns.getPlugin(), "killZoneLocationKey");

    public static boolean isSpawnedMob(Entity entity) {
        return Boolean.TRUE.equals(entity.getPersistentDataContainer().get(questMobKey, PersistentDataType.BOOLEAN));
    }

    public static void setSpawnedMob(Entity entity) {
        entity.getPersistentDataContainer().set(questMobKey, PersistentDataType.BOOLEAN, true);
    }

    public static String getQuestId(Entity entity) {
        return entity.getPersistentDataContainer().get(questIdKey, PersistentDataType.STRING);
    }

    public static Integer getStageId(Entity entity) {
        return entity.getPersistentDataContainer().get(stageIdKey, PersistentDataType.INTEGER);
    }

    public static Integer getIndex(Entity entity) {
        return entity.getPersistentDataContainer().get(mobIndexKey, PersistentDataType.INTEGER);
    }

    /**
     * Gets information about the mob's kill zone
     *
     * @param entity <p>The entity to get data for</p>
     * @return <p>An array containing x,y,z,radius</p>
     */
    public static int[] getKillZoneLocation(Entity entity) {
        return entity.getPersistentDataContainer().get(killZoneLocationKey, PersistentDataType.INTEGER_ARRAY);
    }

    /**
     * Stores quest info to a mob
     *
     * @param entity           <p>The entity to store quest data to</p>
     * @param questId          <p>The id of the quest the mob was spawned for</p>
     * @param killLocationInfo <p>Information about the kill location (x, y, z, range)</p>
     */
    public static void setQuestData(Entity entity, String questId, int stageId, int mobIndex, int[] killLocationInfo) {
        PersistentDataContainer persistentDataContainer = entity.getPersistentDataContainer();
        persistentDataContainer.set(questIdKey, PersistentDataType.STRING, questId);
        persistentDataContainer.set(stageIdKey, PersistentDataType.INTEGER, stageId);
        persistentDataContainer.set(mobIndexKey, PersistentDataType.INTEGER, mobIndex);
        persistentDataContainer.set(killZoneLocationKey, PersistentDataType.INTEGER_ARRAY, killLocationInfo);
    }

    /**
     * Gets the 2d (x,z) distance between two locations
     *
     * @param location1 <p>The first location</p>
     * @param location2 <p>The second location</p>
     * @return <p>The 2d distance between the locations</p>
     */
    public static double get2dDistance(Location location1, Location location2) {
        return Math.sqrt(Math.pow(location2.getX() - location1.getX(), 2) +
                Math.pow(location2.getZ() - location1.getZ(), 2));
    }

}
